var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { createOCRADataInput, dynamicTruncate, hmac, padZeroStart, parseOCRASuite, } from "./helpers";
import { OCRAAlgorithmMap } from "./enums";
/**
 *
 * TOTP: Time-Based One-Time Password Algorithm
 * https://www.rfc-editor.org/rfc/rfc6238
 * @param secret - shared secret between client and server; each HOTP
  generator has a different and unique secret K.
 * @param options.t0 - is the Unix time to start counting time steps, default 0
 * @param options.timestamp - Unix time for which we will generate TOTP
 * @param options.digitsCount - number of digits generated by TOTP, default 6
 * @param options.stepTime - time in seconds how long step is valid, default 30
 * @returns
 */
export function generateTOTP(secret, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let stepTime = options.stepTime || 30;
        let _t0 = options.t0 || 0;
        let stepCounter = Math.floor((options.timestamp - _t0) / 1000 / stepTime);
        return generateHOTP(secret, stepCounter, options);
    });
}
/**
 * HOTP: An HMAC-Based One-Time Password Algorithm
 * https://www.rfc-editor.org/rfc/rfc4226
 * @param secret - shared secret between client and server; each HOTP
  generator has a different and unique secret K.
 * @param counter - integer counter value, the moving factor.  This counter
    MUST be synchronized between the HOTP generator (client)
    and the HOTP validator (server).
 * @param options.digitsCount - length of generated HOTP, default 6
 * @param options.algorithm - algorithm used possible values: sha-1, sha-256, default: sha-1
  
 * @returns
 */
export function generateHOTP(secret, counter, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let hmacResult = yield hmac(secret, counter, options === null || options === void 0 ? void 0 : options.algorithm);
        let digitsCount = (options === null || options === void 0 ? void 0 : options.digitsCount) || 6;
        let codeValue = dynamicTruncate(hmacResult) % Math.pow(10, digitsCount);
        let result = codeValue.toString();
        return padZeroStart(result, digitsCount);
    });
}
/**
 * OCRA: OATH Challenge-Response Algorithm
 * https://www.rfc-editor.org/rfc/rfc6287
 *
 * @param options.question - number or string or ByteArray
 * if string can be alphanumerical or number string eg: "00000000"
 * depends on question type.
 */
export function generateOCRA(secret, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let OCRAConfig = parseOCRASuite(options.suite);
        let dataInput = createOCRADataInput(options, OCRAConfig);
        let hmacResult = yield hmac(secret, dataInput, OCRAAlgorithmMap[OCRAConfig.algorithm]);
        let codeValue = dynamicTruncate(hmacResult) % Math.pow(10, OCRAConfig.digitsCount);
        let result = codeValue.toString();
        return padZeroStart(result, OCRAConfig.digitsCount);
    });
}
